#include "GRIPPlot.h"

int width = 1000;
int height = 1000;
Graph *graph;
DrawGraph *Dgraph;
int dgraph_free = 0;
coord_t boxSize = 1;
const int VIEW_TURN_RATE = 2;
const float ZOOM_FACTOR = .1;
int color = 0;
int dim;
int average_edge_length;
int numOfVert;
int mlistSwitch = 1;   //indicates whether graph is still being processed (1) or final graph is ready (0)
int xspin = 0;
int yspin = 0;
int zspin = 0;
float dist = 1;
Point *baricenter;
size_tt numOfInitVert;
size_tt slowTime = 0;
int createList = 0;
Point *projVect;
Point *nProjVect;
Point *e1;
Point *e2;
Point *e3;
Point *component_offset;  // offset from real origin that will be the origin of each component
int numComponents = 1;
int directed = 0;
int MAX_VERT_NUM = 1000;
EdgeGraphics *edge_graphics;
NodeGraphics *node_graphics;
VisualGraphics *visualGraphics;
RGB_color *bgColor;
//foreground color

//parameters used for drawing arrows
#define ARROW_PIVOT_ANGLE_RATIO 4
float arrow_width;
float arrow_length;

void plotGraphNoDraw(VisualGraphics *_graphics, DrawGraph *_dg, int _width, int _height)
{
  int counter;
  int totalNumVert = 0;
  int totalDiam = 0;
  int maxNumVert = 0;
  int *sortedComp;
  int maxDiam = -1;
  int centerRadius;
  int lastCoord;
  int axis;   // 0 - x, 1 - y, 2 - z
  int posDir; //indicates whether move is in positive or negative direction
  int lastMax;  //records the max diameter of the outer level of components
  int edge;

  //initialize all local variables
  graph      = _graphics->graph;
  Dgraph     = _dg;
  numComponents = _graphics->numComponents;
  directed = _graphics->directed;
  edge_graphics = _graphics->edge_graphics;
  node_graphics = _graphics->node_graphics;

  //calculate the total diameter of all components
  for(counter = 0; counter < numComponents; counter ++)
    {
      totalNumVert += graph[counter].numOfVert;
      maxNumVert = max(maxNumVert, graph[counter].numOfVert);
      totalDiam += Dgraph[counter].diameter;
      debug3("diam of %d is %d\n", counter, Dgraph[counter].diameter);
      if (Dgraph[counter].diameter > maxDiam)
	maxDiam = Dgraph[counter].diameter;
    }

  MAX_VERT_NUM = maxNumVert;

  debug2("total num of vert : %d\n",totalNumVert);
  debug2("total diameter: %d\n",totalDiam);
  //numOfVert  = totalNumVert;
  width      = _width;
  height     = _height;
  dim        = Dgraph->dim;
  numOfInitVert = Dgraph->numOfInitVert;
  Dgraph->edge = 32;
  average_edge_length = sqrt(width * height / totalNumVert);
  debug2("edge is %d\n", Dgraph->edge);

  edge = Dgraph->edge;

  //these variables used for drawing of edges and loops
  arrow_width = 1;
  arrow_length = 3;

  /**************************************************************************
   *  Sort the components by degree. Place the biggest one at the origin and
   *  place the smaller components outside of the diameter of the biggest 
   ***************************************************************************/

  sortedComp = sort_components_by_diam(Dgraph, maxDiam, numComponents);
  component_offset = (Point *)malloc(numComponents * sizeof(Point));
  initialize_Point(&component_offset[sortedComp[0]], 0, 0, 0, 0, dim);
  debug3("offset of component 0: (%d,%d)\n",  get_coord(&component_offset[0], 0),  get_coord(&component_offset[0], 1) );
  centerRadius = ROUND((float)Dgraph[sortedComp[0]].diameter / 2.0);
  //we start off by moving up in y direction
  axis = 1;
  posDir = 1;
  lastCoord = 0;
  lastMax = 0;


  for(counter = 1; counter < numComponents; counter ++)
    {
      int diam = Dgraph[sortedComp[counter]].diameter;
      //keep track of biggest diameter of current level
      if (diam > lastMax)
	lastMax = diam;

      if (posDir)
	lastCoord += ROUND( (diam + 1.0)/2.0);
      else 
	lastCoord -= ROUND( (diam + 1.0)/2.0);

      if (axis == 0)
	initialize_Point(&component_offset[sortedComp[counter]], lastCoord*edge, centerRadius*edge, 0, 0, dim);
      else if (axis == 1)
	initialize_Point(&component_offset[sortedComp[counter]], centerRadius*edge, lastCoord*edge, 0, 0, dim);
      else 
	initialize_Point(&component_offset[sortedComp[counter]], centerRadius*edge, 0, lastCoord*edge, 0, dim);

      debug4("offset of component %d: (%d,%d)\n", sortedComp[counter], get_coord(&component_offset[sortedComp[counter]], 0),  get_coord(&component_offset[sortedComp[counter]], 1));
      // Movement is counter clockwise
      if (posDir && lastCoord > abs(centerRadius))
	{
	  if (centerRadius > 0)  //finished move in positive y, will move in negative x
	    {
	      axis = 0;
	      posDir = 0;
	      lastCoord = 0;
	    }
	  else  //finished moving in positive x, will move in positive y
	    {
	      centerRadius *= -1;  //becomes positive
	      axis = 1;
	      posDir = 1;
	      lastCoord = 0;
	      //centerRadius will be moved at the edge of current level
	      centerRadius += lastMax + 2;
	      lastMax = 0;
	    }
	}
      else  if (!posDir && lastCoord < -abs(centerRadius)) 
	// last movement was in negative direction along 'axis'
	{
	  if (centerRadius > 0)  //finished move in negative x, will move in negative y
	    {
	      centerRadius *= -1;  //becomes negative
	      axis = 0;
	      posDir = 0;
	      lastCoord = 0;
	    }
	  else  //finished moving in negative y, will move in positive x
	    {
	      axis = 0;
	      posDir = 1;
	      lastCoord = 0;
	    }
	}
	
    }
  /********************************************************
   *  End sort components
   *******************************************************/
  
  boxSize = (coord_t)(Dgraph->edge  * 1.5 * centerRadius) + 50;
}


//*******************************************************************
//
//         sorts components by diameter in decreasing order 
//
//*******************************************************************
int * sort_components_by_diam(DrawGraph *dgraph, int maxDiam, int numComponents)
{
  /* NOTE: in the following arrays we ignore values at index 0 */

  int *diam_count;    //diam_count[index] - num of components with diam. index
  int *offset;
  int *comp_processed;  //comp_processed[index] - num of components with diam. index that were processed
  int i;
  int * sorted_components;
  
  diam_count = (int *) malloc ((maxDiam + 1) * sizeof(int));
  offset = (int *) malloc ((maxDiam + 1) * sizeof(int));
  comp_processed = (int *) malloc ((maxDiam + 1) * sizeof(int));

  for(i = 1; i <= maxDiam; i ++)
    {
      diam_count[i] = 0;
      comp_processed[i] = 0;
    }

  for (i = 0; i < numComponents; i ++)
    diam_count[dgraph[i].diameter] ++;

  // make sure offsets are for diameters in descending order
  offset[maxDiam] = 0;
  
  for (i = maxDiam - 1; i >= 1; i --)
    offset[i] = offset[i+1] + diam_count[i+1];

  sorted_components = (int *) malloc (numComponents * sizeof(int));
  for (i = 0; i < numComponents; i ++)
    {
      int diameter = dgraph[i].diameter;
      sorted_components[offset[diameter] + comp_processed[diameter]] = i;
      comp_processed[diameter] ++;
    }

  for (i = 0; i < numComponents; i ++)
    debug4("sorted at pos %d: %d   diam: %d\n", i, sorted_components[i], dgraph[sorted_components[i]].diameter);
  free(diam_count);
  free(offset);
  free(comp_processed);
  return sorted_components;
}


void free_memory()
{
  int i;
  for(i = 0; i < numComponents; i ++)
    {
      free_Point(&component_offset[i]);
    }
  free(component_offset);
}

